             .title        "Jaleco (INES 18) Reader Plugin"


;05/17/07
;Written by BootGod
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h
uaddress:       .equ 00f0h
uaddress_lo:    .equ 00f0h
uaddress_hi:    .equ 00f1h
curbank:    .equ 00e2h
banksleft:    .equ 00e3h
bank0crc:  .equ 00e4h

bank_lo:    .equ 00e8h
bank_hi:    .equ 00e9h

numpages_lo .equ 00eah
numpages_hi .equ 00ebh


temp:    .equ 00ech

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Jaleco SS88006 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 128 >> 4,256 >> 4
             .db 128 >> 3,256 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here

             lda #4
             jsr send_byte	;mirroring is mapper ctrl

;read PRG stuff   
             
do_prg:      lda #2
             sta numpages_hi

             ldy #16
             jsr comp_pbank
             beq got_prg	;is 128K PRG

             asl numpages_hi	;else its 256K PRG
             
got_prg:     lda #0
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #1
             jsr send_byte
             
             lda #0
             sta curbank
             lda numpages_hi
             asl a
             asl a
             asl a
             sta banksleft
             
next_prg:    lda curbank
             jsr wr_pbank

             lda #0		;set up read from $8000
             sta address_lo
             lda #80h
             sta address_hi

             ldy #0		;set counters for 8K dump
             ldx #32

dump_prg:    lda (address),y
             jsr send_byte
             iny
             bne dump_prg
             inc address_hi
             dex
             bne dump_prg
             inc curbank
             dec banksleft
             bne next_prg
                          
;read CHR stuff
          
             
do_chr:      lda #2
             sta numpages_hi

             ldy #128
             jsr comp_cbank
             beq got_chr	;is 128K CHR

             asl numpages_hi	;else its 256K CHR            
             
got_chr:     lda #0
             jsr send_byte
             lda numpages_hi
             jsr send_byte
             lda #2
             jsr send_byte
             
             lda #0
             sta curbank
             lda numpages_hi
             asl a
             asl a
             asl a
             asl a
             asl a
             asl a
             sta banksleft
             
next_chr:    lda curbank
             jsr wr_cbank

             lda #0		;set up PPU to read from $0000
             jsr wr_ppu
             lda 2007h

             ldy #0		;set counters for 1K dump
             ldx #4

dump_chr:    lda 2007h
             jsr send_byte
             iny
             bne dump_chr
             dex
             bne dump_chr
             inc curbank
             dec banksleft
             bne next_chr
             
done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts


;y = bank to compare
;z=same
comp_pbank:  ldx #0
             stx bank_lo	;lower pointer
             sty bank_hi	;upper pointer
             sty banksleft	;# blocks to compare

cb_loop:     lda #0		;init pointers to $8000 & $A000
             sta address_lo
             sta uaddress_lo
             lda #080h
             sta address_hi
             lda #0a0h
             sta uaddress_hi

             lda bank_lo	;write to registers
             jsr wr_pbank

             lda bank_hi
             jsr wr_pbank2

             ldx #32		;set up for 8K comp
             ldy #0

cb_loop2:    lda (address),y
             cmp (uaddress),y
             bne diff_prg
             iny
             bne cb_loop2
             inc address_hi
             inc uaddress_hi
             dex
             bne cb_loop2
             inc bank_lo
             inc bank_hi
             dec banksleft
             bne cb_loop

diff_prg:    rts

;y = bank to compare
;z=same
;for CHR
comp_cbank:  ldx #0
             stx bank_lo	;lower pointer
             sty bank_hi	;upper pointer
             sty banksleft	;# blocks to compare

cc_loop:     lda bank_lo	;write to registers
             jsr wr_cbank

             lda bank_hi
             jsr wr_cbank2

             ldx #4
             lda #0
             sta curbank	;reset current bank

cc_loop2:    ldy #0		;setup PPU
             lda curbank
             sta 2006h
             sty 2006h
             lda 2007h

ql:          lda 2007h		;load 256 bytes for testing
             sta 0300h,y
             iny
             bne ql

             lda curbank	;set up CHR address to compare to
             clc
             adc #4		;comparing @ 400h on top of other bank
             inc curbank

             sta 2006h		
             sty 2006h
             lda 2007h		;garbage read

cc_loop3:    lda 2007h
             cmp 0300h,y
             bne diff_chr
             iny
             bne cc_loop3
             dex
             bne cc_loop2
             inc bank_lo
             inc bank_hi
             dec banksleft
             bne cc_loop

diff_chr:    rts

wr_pbank:    sta temp
             and #0f0h
             lsr a
             lsr a
             lsr a
             lsr a
             sta 08001h
             lda temp
             and #0fh
             sta 08000h
             lda temp
             rts

wr_pbank2:   sta temp
             and #0f0h
             lsr a
             lsr a
             lsr a
             lsr a
             sta 08003h
             lda temp
             and #0fh
             sta 08002h
             lda temp
             rts

wr_cbank:    sta temp
             and #0f0h
             lsr a
             lsr a
             lsr a
             lsr a
             sta 0a001h
             lda temp
             and #0fh
             sta 0a000h
             lda temp
             rts

wr_cbank2:   sta temp
             and #0f0h
             lsr a
             lsr a
             lsr a
             lsr a
             sta 0a003h
             lda temp
             and #0fh
             sta 0a002h
             lda temp
             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
